<?php

namespace QuiversBopis\Components;

use Quivers\Components\SettingService;
use Shopware\Components\Plugin\ConfigReader;
use Shopware\Components\Model\ModelManager;
use Doctrine\DBAL\Connection;
use Enlight_Controller_ActionEventArgs as ActionEventArgs;


class ConfigHelper
{
    const QUIVERSSANDBOX = 'https://api.quiversdemo.com/';
    const QUIVERSLIVE = 'https://api.quivers.com/';
    const bopisStatusCode = 1;
    const shippingSatusCode = 0;
    const mixedStatusCode = 2;
    const errormessagedefault ='Please reload the page and try again. If the issue persists please contact Quivers at 805-250-4555';
    const storepickupcode ="bopispickup";
    const bopismixed ="bopismixed";
    const MiddlewareSANDBOX = 'https://middleware.quiversdemo.com/profile/';
    const MiddlewareLIVE = 'https://middleware.quivers.com/profile/';
    const logEndpoint = 'logs/';
    const inventoryEndpoint = "v1/inventory/getFulfillmentOptions?storefrontId=%s&searchTerm=%s";
    const BopisMethod = "In-Store Pickup";
    const MixedMethod = "BOPIS Mixed";
    const OrderStoreEndpoint = "v1/storage/set?business=%s";
    const storepickupMessage = "NOTE: We are storing this information even if the order is Instore Pickup just in case an order cannot be picked up from the store."; 

    public function __construct(
        $pluginName,
        ConfigReader $configReader,
        ModelManager $modelManager,
        Connection $connection
    )
    {
       $this->shop = Shopware()->Shop();
       $this->config = $configReader->getByPluginName($pluginName, $this->shop);
       $this->modelManager = $modelManager;
       $this->dbalConnection = $modelManager->getConnection();
       $this->connection = $connection;
       $this->admin = Shopware()->Modules()->Admin();
       $this->basket = Shopware()->Modules()->Basket();
       $this->session = Shopware()->Session();
       $this->db = Shopware()->Db();
       $this->front = Shopware()->Container()->get('Front');

    }

    public function BopisActive(){
       return $this->config['QuiversBopisEnabled'];
    }

    public function BopisSandbox(){
        return $this->config['QuiversBopisEnabled'];
    }

    public function BopisEmailFlag(){
        return $this->config['QuiversEmail'];
    }

    public function getCDN(){
        //uncomment on our TEST environment
        // if($this->BopisSandbox()){
        //     //for test env
        //     return "https://d37gifaus44yor.cloudfront.net/bopishosted.js";
        // }
        // else{
        //     // for demo/prod env
        //     return "https://d3su35eq9vvgkf.cloudfront.net/bopishosted.js";
        // }

        //for demo/prod env
        return "https://d3su35eq9vvgkf.cloudfront.net/bopishosted.js";
    }

    public function getAPIbaseURL(){  
        if($this->BopisSandbox()){
            return self::QUIVERSSANDBOX;
        }
        else{
            return self::QUIVERSLIVE;
        }
    }

    public function getMiddlewareURL(){  
        if($this->BopisSandbox()){
            return self::MiddlewareSANDBOX;
        }
        else{
            return self::MiddlewareLIVE;
        }
    }
  
    public function getUPC($sArticle){
        $settingsService = $this->quiversSettings();
        $upc_field = $settingsService->get('upc_field', 'general');
        $upc = "";
        if ($upc_field != null){
            $upc = $sArticle[$upc_field];
        }
        if(!$upc){
            $upc = $sArticle["quivers_upc"];
        }
        return $upc;
    }

    public function getSearchTerm($sArticle){  
        $sku = $sArticle["ordernumber"];
        $upc = $this->getUPC($sArticle);
       
        $searchterm = [
            'upc'=>$upc,
            'sku'=>$sku
        ];
        return $searchterm;

    }

    public function quiversSettings(){  
        $quiverssettingService = new SettingService($this->modelManager);
        return $quiverssettingService;
    }

    public function getQuiversRefId(){  
        $settings = $this->quiversSettings();
        $refId = $settings->get('business_RefId', 'general');
        return $refId;
    }

    public function getAPIkey(){  
        $settings = $this->quiversSettings();
        $quiversAPIKey = $settings->get('api_key', 'general');
        return $quiversAPIKey;
    }

    public function getQuiversEnable(){  
        $settings = $this->quiversSettings();
        $enable = $settings->get('enable_plugin', 'general');
        return $enable;
    }


    public function getMarketPlaceForCurrency() {
        $orderCurrency = Shopware()->Shop()->getCurrency()->getCurrency();
        $settingsService = $this->quiversSettings();
        if($orderCurrency) {
            $marketPlaceId = null;
            switch (strtolower($orderCurrency)) {
                case 'usd':
                    $marketPlaceId = $settingsService->get('market_places', 'tax_settings', 'USD');
                    break;
                case 'eur':
                    $marketPlaceId =$settingsService->get('market_places', 'tax_settings', 'EUR');
                    break;
                case 'cad':
                    $marketPlaceId = $settingsService->get('market_places', 'tax_settings', 'CAD');
                case 'aud':
                    $marketPlaceId = $settingsService->get('market_places', 'tax_settings', 'AUD');
                    break;
                case 'jpy':
                    $marketPlaceId = $settingsService->get('market_places', 'tax_settings', 'JPY');
                    break;
                case 'gbp':
                    $marketPlaceId = $settingsService->get('market_places', 'tax_settings', 'GBP');
                    break;
            }
            return $marketPlaceId;
        }
    } 

    public function CallApi($method, $url, $headers = false, $payload = [], $timeout = 10){

        $curl = curl_init();
        switch ($method)
		{
			case "POST":
				curl_setopt($curl, CURLOPT_POST, 1);
				if ($payload)
					curl_setopt($curl, CURLOPT_POSTFIELDS, $payload);
				break;
                
			default:
				if ($payload)
					$url = sprintf("%s?%s", $url, http_build_query($payload));
		}

		curl_setopt($curl, CURLOPT_URL, $url);
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, $timeout - 1);
		curl_setopt($curl, CURLOPT_TIMEOUT, $timeout);
		if ($headers) {
			foreach ($headers as $header) {
				curl_setopt( $curl, CURLOPT_HTTPHEADER, $header);
			}
		}

		try{
			$result = curl_exec($curl);
		}
		catch(Exception $e) {
            $res = [
                "result"=>[
                    "URL"=>$url,
                    "Payload"=>$payload,
                    "headers"=>$headers,
                    "Error"=>$e->getMessage()
				]
            ];
			return $res;
		}

		$httpcode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
		$res['result'] = $result;
		$res['httpcode'] = @$httpcode;
		return $res;

    }

    public function addBopisData($data, $table, $basketId)
    {
        $identifier = [
            "basketID"=>$basketId
        ];
        $this->connection->update($table, $data, $identifier);

    }
    
    public function isSaleable($sArticle)
    { 
        $storeFrontId = $this->getMarketPlaceForCurrency();
        $searchTerm = $this->getSearchTerm($sArticle);
        $searchkey =null;
        $postalcodes = [];

        if ($this->BopisActive() && isset($storeFrontId)) {
            try {
                if($searchTerm['upc']==null){
                    $searchkey =$searchTerm['sku'];
                    $url= $this->getApiBaseUrl().'v1/inventory/getFulfillmentOptions?storefrontId='.$storeFrontId.'&searchTerm='.$searchTerm['sku'];
                }
                else{
                    $searchkey =$searchTerm['upc'];
                    $url=$this->getApiBaseUrl().'v1/inventory/getFulfillmentOptions?storefrontId='.$storeFrontId.'&searchTerm='.$searchTerm['upc'];
                }
                $response = $this->CallAPI(
                    "GET",
                    $url
                );
              
                if($response['httpcode'] == 200){
                    $resultdata =json_decode($response['result'], true);
                    if(isset($searchTerm['sku']) && $searchTerm['upc']!=null &&  isset($resultdata['result']) && empty($resultdata['result'])){
                        $searchkey =$searchTerm['sku'];
                        $url= $this->getApiBaseUrl().'v1/inventory/getFulfillmentOptions?storefrontId='.$storeFrontId.'&searchTerm='.$searchTerm['sku'];
                        $response = $this->CallAPI(
                            "GET",
                            $url
                        );
                        if($response['httpcode'] == 200){
                            foreach ($resultdata['result'] as $store ) {
                                array_push($postalcodes,$store['address']['postalCode']);
                            }
                            $result =[
                                'postalcodes'=>json_encode($postalcodes),
                                'stores'=>$resultdata['result'],
                                'searchterm'=>$searchkey
                            ];
                            
                            return $result;
                        }
                    }
                    else{
                        foreach ($resultdata['result'] as $store ) {
                            array_push($postalcodes,$store['address']['postalCode']);
                        }
                        $result =[
                            'postalcodes'=>json_encode($postalcodes),
                            'stores'=>$resultdata['result'],
                            'searchterm'=>$searchkey
                        ];
                        return $result;
                    }
                }
                else {                    
                       return $result =[
                            'postalcodes'=>null,
                            'stores'=>null,
                            'searchterm'=>$searchkey
                        ];
                   
                }
            }
            catch(Exception $e) {
            }
        }else {
           return $result=[
                'storefrontId'=>null,
                'stores'=>null,
                'searchterm'=>null

            ];
        }
    }

    public function getModalData($searchTerm,$storeFrontId) 
    {
        $stores = [];
        try {
            $url= $this->getApiBaseUrl().sprintf(self::inventoryEndpoint,$storeFrontId,$searchTerm);
            $response = $this->CallAPI(
                "GET",
                $url
            );
            if($response['httpcode'] == 200){
                $resultdata =json_decode($response['data'], true);
                return json_encode($resultdata['result']);
            }else {
                if($response['httpcode']==500 || $response['httpcode']==502) {
                    throw new \Exception("Internal server error");
                } else {
                    $message = isset(json_decode($response['data'], true)['error']['message'])?json_decode($response['data'], true)['error']['message']:json_decode($response['data'], true)['error'];
                    throw new \Exception($message);
               }
            }
        }
        catch(\Exception $e) {
        }
    }


    public function addProdBopisData(ActionEventArgs $args)
    {
        $controller = $args->getSubject();
        $action = $controller->Request()->getActionName();
        $request = $controller->Request();
        $view = $controller->View();

        $allowedActions = ['ajaxCart'];
        if (!in_array($action, $allowedActions, true)) {
            return;
        }

        //Get BOPIS data 
        $uuid = $this->session->get('sessionId');
        $userPostalcode = $request->getParam('userPostalcode');
        $searchterm = $request->getParam('searchterm');
        $store = $request->getParam('store')!=null?json_decode($request->getParam('store'), true):null;
        $address = isset($store['address'])!=null?$store['address']:null;
        $shipping = $request->getParam('shipping')!=null? $request->getParam('shipping'):null;
        $marketplace = $request->getParam('marketplace');

        // Add BOPIS data in the db
        $article = $view->getAssign('sBasket'); 
        $basketItems = $article["content"];
        $searchtermArray = [];
        foreach ($basketItems as $basketItem) {
            $sku = $basketItem["ordernumber"];
            $upc = $this->getUPC($basketItem);
            $articleId= $basketItem["articleID"];
            $id = $basketItem["id"];
            $currentProduct = $searchterm != null && ($searchterm == $sku || $searchterm == $upc) ? true: false;
            if ($searchterm != null && $currentProduct)
            {
                $cartItems = [
                    'userlocation'=>$userPostalcode,
                    'businessRefId' => isset($store['businessRefId'])?$store['businessRefId']:null,
                    'searchterm' => $searchterm, 
                    'businessName' => isset($store['businessName'])?$store['businessName']:null, 
                    'bopis_city' =>  isset($store['businessRefId'])?$address['city']:null, 
                    'bopis_storepostalcode' => isset($address['postalCode'])?$address['postalCode']:null,
                    'bopis_state' => isset($address['region']['abbreviation'])?$address['region']['abbreviation']:null,
                    'bopis_country' => isset($address['country']['abbreviations']['two'])?$address['country']['abbreviations']['two']:null, 
                    'curbsidepick' =>$shipping== 2 ? true : false, 
                    'instorepickup' =>$shipping == 1 ? true : false,
                    'bopis_item' => isset($store['item']['productVariantId']) && $store['item']['productVariantId']!=null? $store['item']['productVariantId']:null
                ];
                //insert in db
                $this->addBopisData($cartItems, 's_order_basket_attributes', $id);
                //save stores in the session to use later
                $stores = $request->getParam('stores');
                $key = 'QuiversStores-'.$articleId;
                $this->session[$key] = $stores;

            }
        }

    }


    public function addProdStores(ActionEventArgs $args, $cartAction = false)
    {
        $controller = $args->getSubject();
        $view = $controller->View();
        $sBasket = $view->getAssign('sBasket');
       
        foreach($sBasket['content'] as $key=>$basketItem){
            if($basketItem['__s_order_basket_attributes_businessrefid']!=null){
                $articleId= $basketItem["articleID"];
                $storekey = 'QuiversStores-'.$articleId;
                $stores = $this->session[$storekey];
                if($stores==null || empty($stores)) {
                    $storeFrontId = $this->getMarketPlaceForCurrency();
                    $stores = $this->getModalData(
                        $basketItem['__s_order_basket_attributes_searchterm'],
                        $storeFrontId
                    );
                
                }
                $sBasket['content'][$key]['QuiversStores']=  $stores;
            }  
        }
    
        $view->assign('sBasket', $sBasket);
    }

    public function getsiteBaseUrl(){
        $shop = Shopware()->Shop();
        $mainShop = $shop->getMain() !== null ? $shop->getMain() : $shop;
        return $mainShop->getBaseUrl();
    }

    public function getShippingMethods(ActionEventArgs $args)
    {
        $allMethods = [];
        $shpMethods = [];
        $quiversMethods = [];
        $storePickup = [];

        $controller = $args->getSubject();
        $view = $controller->View();

        $dispatchId = $this->session['sDispatch'];
        $dispatches = $view->getAssign('sDispatches');
        foreach($dispatches as $dispatch){
            $this->session['sDispatch'] = $dispatch["id"];
            $shippingcosts = $this->getShippingCosts();    
            if($dispatch['name']!=self::BopisMethod && $dispatch['name']!=self::MixedMethod)
            {
                $shpMethods[] = array('id'=>$dispatch["id"],'name'=>$dispatch["name"],'price'=>$shippingcosts['netto'],'shippingcosts'=>$shippingcosts);
            }
            else if($dispatch['name']==self::MixedMethod)
            {
                $quiversMethods[] = array('id'=>$dispatch["id"],'name'=>$dispatch["name"],'price'=>$shippingcosts['netto'],'shippingcosts'=>$shippingcosts);
            }  

            if($dispatch['name']==self::BopisMethod){
                $storePickup[] = array('id'=>$dispatch["id"],'name'=>$dispatch["name"],'price'=>$shippingcosts['netto'],'shippingcosts'=>$shippingcosts);
            }
            $allMethods[] = array('id'=>$dispatch["id"],'name'=>$dispatch["name"],'price'=>$shippingcosts['netto'],'shippingcosts'=>$shippingcosts);
        }

        $this->session['sDispatch'] = $dispatchId;

        //set methods in view
        $view->assign('shpMethods', $shpMethods);
        $view->assign('quiversMethods', $quiversMethods);
        $view->assign('allMethods', $allMethods);
        $view->assign('storePickup', $storePickup);
        //set methods in session
        $this->session['shpMethods'] = $shpMethods;
        $this->session['quiversMethods'] = $quiversMethods;
        $this->session['allMethods'] = $allMethods;
        $this->session['storePickup'] = $storePickup;    
    }


    public function setShippingMethod(ActionEventArgs $args)
    {
        $controller = $args->getSubject();
        $view = $controller->View();
        $action = $controller->Request()->getActionName();
        $sBasket = $view->getAssign('sBasket');

        $cartDispatches = $this->session['shpMethods'];
        $mixedMethods = $this->session['quiversMethods'][0];
        $storePickup = $this->session['storePickup'][0];

        $shipMethod = null;
        $finalMethod = null;
        if(!$this->session['shipMethodId'] && $cartDispatches){
            $this->session['shipMethodId'] = $cartDispatches[0]['id'];
        }
        $changeShipMethodId = $this->session['QuiverssDispatch'];
        if($changeShipMethodId){
            $this->session['shipMethodId'] = $changeShipMethodId;
        }
        
        if($this->session['shipMethodId']){
            $shipMethod = $this->getShippingMethodById($this->session['shipMethodId'], $cartDispatches);
        }

        if ($sBasket['bopis_shipping_status'] == self::shippingSatusCode){
            $method= $this->getShippingMethodById($this->session['sDispatch'], $cartDispatches);
            if ($method){
                $totalShipamount = $method['price'];
                $shipMethod= $method;
                $finalMethod = $method;
            }
            else{
                $this->session['sDispatch'] = $this->session['shipMethodId'];
                $totalShipamount = $shipMethod['price'];  
                $finalMethod = $shipMethod;  
            }

            
        }
        else if ($sBasket['bopis_shipping_status'] == self::bopisStatusCode){
            $this->session['sDispatch'] = $storePickup['id'];
            $totalShipamount = 0;
            $finalMethod = $storePickup;

        }
        else if ($sBasket['bopis_shipping_status'] == self::mixedStatusCode){
            $this->session['sDispatch'] = $mixedMethods['id'];
            $totalShipamount = $shipMethod['price'];
            $finalMethod = $mixedMethods;
            //set the total shipping ampunt in the db
            if($action!='cart'){
                $this->setMixedShipCost($totalShipamount, $view);
            }
            $netAmount = $view->getAssign('sAmount');
            $view->assign('sAmountWithTax', ($totalShipamount+$netAmount));
            $view->assign('sAmount', ($netAmount+$totalShipamount));
        }


        $view->assign('QuiversShipCost', $totalShipamount);
        $view->assign('QuiversItemDispatches', $cartDispatches);
        $view->assign('QuiversStorePickup', $storePickup);
        $view->assign('QuiversBopisMixed', $this->session['quiversMethods']);

        //split shipping amount at line item level
        $shipped_qty = $sBasket['quivers_shipped_items'];
        $approx_ship_amounts = [];
        if ($shipped_qty>0){
            $approx_ship_amounts = $this->splitShippingLineItem($totalShipamount, $shipped_qty);
        }
        $i= 0;
        $title = "";
        foreach($sBasket['content'] as $basketItem){
            $data = [];
            $id = $basketItem['id'];
            if(!$basketItem['__s_order_basket_attributes_businessrefid']){
                if($shipMethod!=null && strpos($title, $shipMethod['name']) === false && $sBasket['bopis_shipping_status'] == self::mixedStatusCode){
                    if(!$title){
                        $title = "-".$shipMethod['name'];
                    }
                    else{
                        $title = $title.",".$shipMethod['name'];
                    }    
                }
                $data = [
                    "bopis_shipping_name"=> ($shipMethod!=null) ? $shipMethod['name'] : "",
                    "bopis_shipping_id"=> ($shipMethod!=null) ? $shipMethod['id'] : "",
                    "bopis_shipping_fee"=>$approx_ship_amounts[$i]
                ];
                $i++;
            }
            else if($basketItem['__s_order_basket_attributes_businessrefid']){
                if(strpos($title, $storePickup['name']) === false && $sBasket['bopis_shipping_status'] == self::mixedStatusCode){
                    if(!$title){
                        $title = "-".$storePickup['name'];
                    }
                    else{
                        $title = $title.",".$storePickup['name'];
                    }    
                }

                $data = [
                    "bopis_shipping_name"=>$storePickup["name"],
                    "bopis_shipping_id"=>$storePickup["id"],
                    "bopis_shipping_fee"=>$storePickup["price"]
                ];

            }
            if($data!=[]){
                $this->addBopisData($data, 's_order_basket_attributes', $id);
            }
        }

        
        $this->session['QuiversShippingTitle']= $title;
        $this->session['QuiversTotalShipping']= $totalShipamount;
        $view->assign("title", $title);
        $view->assign("sDispatch", $finalMethod);
        $view->assign('sShippingcosts', $totalShipamount);
    }



    public function getShippingMethodById($dispatchId, $dispatches){
        foreach($dispatches as $dispatch){
            if($dispatchId == $dispatch['id']){
                return $dispatch;
            }
        }
        return null;

    }

    public function setMixedShipCost($totalShipamount){
        $orderVariables = $this->session['sOrderVariables'];
        if($orderVariables != null){
            $orderVariables['sBasket']["sShippingcostsNet"] = $totalShipamount;
            $orderVariables['sBasket']["sShippingcostsWithTax"] = $totalShipamount;
            $orderVariables['sBasket']["AmountNumeric"] += $totalShipamount;
            $orderVariables['sBasket']["AmountNetNumeric"] += $totalShipamount;
            $orderVariables['sBasket']["AmountWithTaxNumeric"] += $totalShipamount;
        }
    }

    public function splitShippingLineItem($amount, $item_count){
        # divide shipping by item_count, truncate it to 2 decimal places
        $amount= (float)$amount;
        $item_count = (int)$item_count;
        $apprx_per_item_amount = (float)$amount/$item_count;
        $apprx_per_item_str = floatval(substr($apprx_per_item_amount, 0, strpos($apprx_per_item_amount, '.') + 3));
        
        # as we are truncating, difference woud always be +ve and needs to be added
        # also this won't be greater than 0.01 times the item count
        $positive_diff = round($amount - ((float)$apprx_per_item_str*$item_count),2);
        $approx_ship_amounts = [];
        for ($x = 1; $x <= $item_count; $x++) {
            array_push($approx_ship_amounts, $apprx_per_item_str);
        }
        
        # add 0.01 of the positive_diff in each approx_ship_amount until it reduces to 0
        $idx = 0;
        while($positive_diff > 0) {
            $approx_ship_amounts[$idx] = (string)(round((float)$approx_ship_amounts[$idx] + 0.01,2));
            $positive_diff = $positive_diff-0.01;
            $idx = $idx + 1;        
        }
        return $approx_ship_amounts;  
    
    }


    public function getBasketdataToSetOrder($items)
    {
        $cartdata = [];

        foreach($items as $item){
            $cartItems = [
                'id' => $item['articleID'],
                'qty'=>$item['quantity'],
                'businessRefId' => $item['__s_order_basket_attributes_businessrefid'],
                'searchterm' => $item['__s_order_basket_attributes_searchterm'], 
                'curbsidepick' => $item['__s_order_basket_attributes_curbsidepick'], 
                'instorepickup' => $item['__s_order_basket_attributes_instorepickup']

            ];
            array_push($cartdata,$cartItems);
            
        }
        return $cartdata;
    }

    //************************** */
    //Shipping costs function written from checkout controller
    //************************** */
    public function getShippingCosts()
    {
        $country = $this->getSelectedCountry();
        $payment = $this->getSelectedPayment();
        if (empty($country) || empty($payment)) {
            return ['brutto' => 0, 'netto' => 0];
        }
        $shippingcosts = $this->admin->sGetPremiumShippingcosts($country);
        return empty($shippingcosts) ? ['brutto' => 0, 'netto' => 0] : $shippingcosts;
    }


    public function getSelectedCountry()
    {
        if (!empty($this->view->sUserData['additional']['countryShipping'])) {
            $this->session['sCountry'] = (int) $this->view->sUserData['additional']['countryShipping']['id'];
            $this->session['sArea'] = (int) $this->view->sUserData['additional']['countryShipping']['areaID'];

            return $this->view->sUserData['additional']['countryShipping'];
        }
        $countries = $this->getCountryList();
        if (empty($countries)) {
            unset($this->session['sCountry']);

            return false;
        }
        $country = reset($countries);
        $this->session['sCountry'] = (int) $country['id'];
        $this->session['sArea'] = (int) $country['areaID'];
        $this->view->sUserData['additional']['countryShipping'] = $country;
        return $country;
    }

    public function getCountryList()
    {
        return $this->admin->sGetCountryList();
    }

    public function getPayments()
    {
        return $this->admin->sGetPaymentMeans();
    }


    public function getSelectedPayment()
    {
        $paymentMethods = $this->getPayments();

        if (!empty($this->view->sUserData['additional']['payment'])) {
            $payment = $this->view->sUserData['additional']['payment'];
        } elseif (!empty($this->session['sPaymentID'])) {
            $payment = $this->admin->sGetPaymentMeanById($this->session['sPaymentID'], $this->view->sUserData);
        }

        if ($payment && !$this->checkPaymentAvailability($payment, $paymentMethods)) {
            $payment = null;
        }

        $paymentClass = $this->admin->sInitiatePaymentClass($payment);
        if ($payment && $paymentClass instanceof \ShopwarePlugin\PaymentMethods\Components\BasePaymentMethod) {
            $data = $paymentClass->getCurrentPaymentDataAsArray(Shopware()->Session()->sUserId);
            $payment['validation'] = $paymentClass->validate($data);
            if (!empty($data)) {
                $payment['data'] = $data;
            }
        }

        if (!empty($payment)) {
            return $payment;
        }

        if (empty($paymentMethods)) {
            unset($this->session['sPaymentID']);

            return false;
        }

        $payment = $this->getDefaultPaymentMethod($paymentMethods);

        $this->session['sPaymentID'] = (int) $payment['id'];
        $this->front->Request()->setPost('sPayment', (int) $payment['id']);
        $this->admin->sUpdatePayment();

        //if customer logged in and payment switched to fallback, display cart notice. Otherwise anonymous customers will see the message too
        if (Shopware()->Session()->sUserId) {
            $this->flagPaymentBlocked();
        }

        return $payment;
    }

    public function flagPaymentBlocked()
    {
        $this->view->assign('paymentBlocked', true);
    }

    private function checkPaymentAvailability($currentPayment, $payments)
    {
        foreach ($payments as $availablePayment) {
            if ($availablePayment['id'] === $currentPayment['id']) {
                return true;
            }
        }

        return false;
    }
    private function getDefaultPaymentMethod(array $paymentMethods)
    {
        $payment = null;

        foreach ($paymentMethods as $paymentMethod) {
            if ($paymentMethod['id'] == Shopware()->Config()->offsetGet('defaultpayment')) {
                $payment = $paymentMethod;
                break;
            }
        }

        if (!$payment) {
            $payment = reset($paymentMethods);
        }

        return $payment;
    }


}
