<?php

namespace QuiversBopis\Setup;

use Shopware\Bundle\AttributeBundle\Service\CrudService;
use Shopware\Components\Model\ModelManager;
use Doctrine\DBAL\Connection;

class Installer
{
    /**
     * @var ModelManager
     */
    private $modelManager;

    /**
     * @var CrudService
     */
    private $attributeCrudService;


    public function __construct(
        ModelManager $modelManager,
        CrudService $attributeCrudService,
        Connection $connection
    ) {
        $this->modelManager = $modelManager;
        $this->attributeCrudService = $attributeCrudService;
        $this->connection = $connection;
        $this->quiversMethodsIds = [];
    }

    /**
     * @throws InstallationException
     *
     * @return bool
     */
    public function install()
    {
        $this->createAttributes();
        $this->createShippingMethods();
        $this->AssociateCountriesWithMethods();
        $this->AssociatePaymentsWithMethods();
        $this->installMailTemplate();
        return true;
    }


    /**
     * Install attributes when Bopis plugin is installed.
    */
    private function createAttributes()
    {
        //Shipping attributes
        $this->attributeCrudService->update('s_order_details_attributes', "bopis_shipping_id", 'string', [
            'custom' => true,
            'supportText' => '',
            'label' => "Bopis Shipping Id",
            'displayInBackend' => false,
        ], null, true, null);
    
        $this->attributeCrudService->update('s_order_details_attributes', "bopis_shipping_name", 'string', [
            'custom' => true,
            'supportText' => '',
            'label' => "Bopis Shipping Name",
            'displayInBackend' => false,
        ], null, true, null);

        $this->attributeCrudService->update('s_order_details_attributes', "bopis_shipping_fee", 'float', [
            'custom' => true,
            'supportText' => '',
            'label' => "Bopis Shipping Fee",
            'displayInBackend' => false,
        ], null, true, 0.0);

        //Bopis Data sttributes
        $this->attributeCrudService->update('s_order_details_attributes', "userlocation", 'string', [
            'custom' => true,
            'supportText' => '',
            'label' => "Bopis userlocation",
            'displayInBackend' => false,
        ], null, true, null);
        $this->attributeCrudService->update('s_order_details_attributes', "searchterm", 'string', [
            'custom' => true,
            'supportText' => '',
            'label' => "Bopis searchterm",
            'displayInBackend' => false,
        ], null, true, null);
        $this->attributeCrudService->update('s_order_details_attributes', "businessRefId", 'string', [
            'custom' => true,
            'supportText' => '',
            'label' => "Bopis businessRefId",
            'displayInBackend' => false,
        ], null, true, null);
        $this->attributeCrudService->update('s_order_details_attributes', "businessName", 'string', [
            'custom' => true,
            'supportText' => '',
            'label' => "Bopis businessName",
            'displayInBackend' => false,
        ], null, true, null);
        $this->attributeCrudService->update('s_order_details_attributes', "bopis_city", 'string', [
            'custom' => true,
            'supportText' => '',
            'label' => "Bopis city",
            'displayInBackend' => false,
        ], null, true, null);
        $this->attributeCrudService->update('s_order_details_attributes', "bopis_storepostalcode", 'string', [
            'custom' => true,
            'supportText' => '',
            'label' => "Bopis storepostalcode",
            'displayInBackend' => false,
        ], null, true, null);
        $this->attributeCrudService->update('s_order_details_attributes', "bopis_state", 'string', [
            'custom' => true,
            'supportText' => '',
            'label' => "Bopis state",
            'displayInBackend' => false,
        ], null, true, null);
        $this->attributeCrudService->update('s_order_details_attributes', "bopis_country", 'string', [
            'custom' => true,
            'supportText' => '',
            'label' => "Bopis country",
            'displayInBackend' => false,
        ], null, true, null);
        $this->attributeCrudService->update('s_order_details_attributes', "instorepickup", 'boolean', [
            'custom' => true,
            'supportText' => '',
            'label' => "instorepickup",
            'displayInBackend' => false,
        ], null, true, 0);
        $this->attributeCrudService->update('s_order_details_attributes', "curbsidepick", 'boolean', [
            'custom' => true,
            'supportText' => '',
            'label' => "curbsidepick",
            'displayInBackend' => false,
        ], null, true, 0);
        $this->attributeCrudService->update('s_order_details_attributes', "bopis_item", 'string', [
            'custom' => true,
            'supportText' => '',
            'label' => "Bopis item",
            'displayInBackend' => false,
        ], null, true, null);

        //Order Level Attributes
        $this->attributeCrudService->update('s_order_attributes', "bopis_shipping_status", 'integer', [
            'custom' => true,
            'label' => "Bopis Shipping status",
            'displayInBackend' => false,
        ], null, true, 0);

        $this->modelManager->generateAttributeModels(['s_order_details_attributes', 's_order_attributes']);
    }

    private function createShippingMethods()
    {
        $sql = $this->connection->createQueryBuilder()
        ->select('*')
        ->from('s_premium_dispatch')
        ->where('name = :name')
        ->setParameter('name', 'In-Store Pickup')
        ->execute()->fetchAll();

        if($sql){
            $this->quiversMethodsIds[$sql[0]['name']] = $sql[0]['id'];
            $data = [
                "active" => 1,
            ];
            $identifier = [
                'id'=>$sql[0]['id']
            ];
            $this->connection->update("s_premium_dispatch", $data, $identifier);    

        }
        if(!$sql){
            $data = [
                "name" => "In-Store Pickup",
                "description" => "",
                "type" => 0,
                "active" => 1,
                "calculation"=>3,
            ];
            $this->connection->insert("s_premium_dispatch", $data);    
        }

        
        $sql = $this->connection->createQueryBuilder()
        ->select('*')
        ->from('s_premium_dispatch')
        ->where('name = :name')
        ->setParameter('name', 'BOPIS Mixed')
        ->execute()->fetchAll();
        if($sql){
            $this->quiversMethodsIds[$sql[0]['name']] = $sql[0]['id'];
            $data = [
                "active" => 1,
            ];
            $identifier = [
                'id'=>$sql[0]['id']
            ];
            $this->connection->update("s_premium_dispatch", $data, $identifier);    

        }

        if(!$sql){
            $data = [
                "name" => "BOPIS Mixed",
                "description" => "",
                "type" => 0,
                "active" => 1,
                "calculation"=>3,
            ];
            $this->connection->insert("s_premium_dispatch", $data);     
        }

    }


    private function AssociateCountriesWithMethods()
    {
        try{
            $countries = $this->connection->createQueryBuilder()
            ->select('id')
            ->from('s_core_countries')
            ->execute()->fetchAll();

            if(!$this->quiversMethodsIds){
                $mixed = $this->connection->createQueryBuilder()
                ->select('*')
                ->from('s_premium_dispatch')
                ->where('name = :name')
                ->setParameter('name', 'BOPIS Mixed')
                ->execute()->fetchAll();
                $this->quiversMethodsIds[$mixed[0]['name']] = $mixed[0]['id'];
    
                $storePickup = $this->connection->createQueryBuilder()
                ->select('*')
                ->from('s_premium_dispatch')
                ->where('name = :name')
                ->setParameter('name', 'In-Store Pickup')
                ->execute()->fetchAll();
                $this->quiversMethodsIds[$storePickup[0]['name']] = $storePickup[0]['id'];
    
            }
    
            $countriesStorePickup = [];
            $countriesMixed = [];
            $dipatchId = $this->quiversMethodsIds['In-Store Pickup'];
            $quiversMethods = $this->connection->createQueryBuilder()
            ->select('countryID')
            ->from('s_premium_dispatch_countries')
            ->where('dispatchID = :dispatchID')
            ->setParameter('dispatchID', $dipatchId)
            ->execute()->fetchAll();
    
            foreach($quiversMethods as $quiversMethod){
                array_push($countriesStorePickup, $quiversMethod['countryID']);
    
            }
    
            $dipatchId = $this->quiversMethodsIds['BOPIS Mixed'];
            $quiversMethods = $this->connection->createQueryBuilder()
            ->select('countryID')
            ->from('s_premium_dispatch_countries')
            ->where('dispatchID = :dispatchID')
            ->setParameter('dispatchID', $dipatchId)
            ->execute()->fetchAll();
    
            foreach($quiversMethods as $quiversMethod){
                array_push($countriesMixed, $quiversMethod['countryID']);
    
            }
    
            foreach($countries as $country){
                if(array_search($country['id'], $countriesStorePickup) === false){
                    $data = [
                        "dispatchID" => $this->quiversMethodsIds['In-Store Pickup'],
                        "countryID" => $country['id']
                    ];
                    $this->connection->insert("s_premium_dispatch_countries", $data);     
        
                }
                if(array_search($country['id'], $countriesMixed) === false){
                    $data = [
                        "dispatchID" => $this->quiversMethodsIds['BOPIS Mixed'],
                        "countryID" => $country['id']
                    ];
                    $this->connection->insert("s_premium_dispatch_countries", $data);     
        
                }
            }
    
        }catch (\Exception $e) {
        }

    }

    private function AssociatePaymentsWithMethods()
    {
        try{
            $payments = $this->connection->createQueryBuilder()
            ->select('id')
            ->from('s_core_paymentmeans')
            ->execute()->fetchAll();

            $paymentsStorePickup = [];
            $paymentsMixed = [];

            $dipatchId = $this->quiversMethodsIds['In-Store Pickup'];
            $quiversMethods = $this->connection->createQueryBuilder()
            ->select('paymentID')
            ->from('s_premium_dispatch_paymentmeans')
            ->where('dispatchID = :dispatchID')
            ->setParameter('dispatchID', $dipatchId)
            ->execute()->fetchAll();

            foreach($quiversMethods as $quiversMethod){
                array_push($paymentsStorePickup, $quiversMethod['paymentID']);

            }

            $dipatchId = $this->quiversMethodsIds['BOPIS Mixed'];
            $quiversMethods = $this->connection->createQueryBuilder()
            ->select('paymentID')
            ->from('s_premium_dispatch_paymentmeans')
            ->where('dispatchID = :dispatchID')
            ->setParameter('dispatchID', $dipatchId)
            ->execute()->fetchAll();

            foreach($quiversMethods as $quiversMethod){
                array_push($paymentsMixed, $quiversMethod['paymentID']);

            }

            foreach($payments as $payment){
                if(array_search($payment['id'], $paymentsStorePickup) === false){
                    $data = [
                        "dispatchID" => $this->quiversMethodsIds['In-Store Pickup'],
                        "paymentID" => $payment['id']
                    ];
                    $this->connection->insert("s_premium_dispatch_paymentmeans", $data);     
        
                }
                if(array_search($payment['id'], $paymentsMixed) === false){
                    $data = [
                        "dispatchID" => $this->quiversMethodsIds['BOPIS Mixed'],
                        "paymentID" => $payment['id']
                    ];
                    $this->connection->insert("s_premium_dispatch_paymentmeans", $data);     
        
                }
            }
        }
        catch (\Exception $e) {
        }
    }



    private function installMailTemplate()
    {
        $defaultContext = $this->connection->createQueryBuilder()
                        ->select('context')
                        ->from('s_core_config_mails')
                        ->where('name = :name')
                        ->setParameter('name', 'sOrder')
                        ->execute()->fetchAll();
        $data = [
            "`frommail`" => '{config name=mail}',
            "`fromname`" => '{config name=shopName}',
            "`subject`" => 'Your order at the {config name=shopName}',
            "`content`" => '{include file="string:{config name=emailheaderplain}"}
            Dear {$billingaddress.salutation|salutation} {$billingaddress.lastname},

            Thank you for your order at {config name=shopName} (Number: {$sOrderNumber}) on {$sOrderDay} at {$sOrderTime}.
            Information on your order:
            
            Pos.  Art.No.               Description                                      Quantities       Price       Total
            {foreach item=details key=position from=$sOrderDetails}
            {{$position+1}|fill:4}  {$details.ordernumber|fill:20}  {$details.articlename|fill:49}  {$details.quantity|fill:6}  {$details.price|padding:8|currency|unescape:"htmlall"}      {$details.amount|padding:8|currency|unescape:"htmlall"}
            {/foreach}
            
            Shipping costs: {$sShippingCosts|currency|unescape:"htmlall"}
            Net total: {$sAmountNet|currency|unescape:"htmlall"}
            {if !$sNet}
            {foreach $sTaxRates as $rate => $value}
            plus {$rate|number_format:0}% VAT {$value|currency|unescape:"htmlall"}
            {/foreach}
            Total gross: {$sAmount|currency|unescape:"htmlall"}
            {/if}
            
            Selected payment type: {$additional.payment.description}
            {$additional.payment.additionaldescription}
            {if $additional.payment.name == "debit"}
            Your bank connection:
            Account number: {$sPaymentTable.account}
            BIN:{$sPaymentTable.bankcode}
            Bank name: {$sPaymentTable.bankname}
            Bank holder: {$sPaymentTable.bankholder}
            
            We will withdraw the money from your bank account within the next days.
            {/if}
            {if $additional.payment.name == "prepayment"}
            
            Our bank connection:
            Account: ###
            BIN: ###
            {/if}
            
            
            Selected shipping type: {$sDispatch.name}
            {$sDispatch.description}
            
            {if $sComment}
            Your comment:
            {$sComment}
            {/if}
            
            Billing address:
            {$billingaddress.company}
            {$billingaddress.firstname} {$billingaddress.lastname}
            {$billingaddress.street} {$billingaddress.streetnumber}
            {if {config name=showZipBeforeCity}}{$billingaddress.zipcode} {$billingaddress.city}{else}{$billingaddress.city} {$billingaddress.zipcode}{/if}
            {$additional.country.countryname}
            
            Shipping address:
            {$shippingaddress.company}
            {$shippingaddress.firstname} {$shippingaddress.lastname}
            {$shippingaddress.street} {$shippingaddress.streetnumber}
            {if {config name=showZipBeforeCity}}{$shippingaddress.zipcode} {$shippingaddress.city}{else}{$shippingaddress.city} {$shippingaddress.zipcode}{/if}\n
            {$additional.countryShipping.countryname}
            
            {if $billingaddress.ustid}
            Your VAT-ID: {$billingaddress.ustid}
            In case of a successful order and if you are based in one of the EU countries, you will receive your goods exempt from turnover tax.{/if}
            
            If you have any questions, do not hesitate to contact us.
            
            {include file="string:{config name=emailfooterplain}"}',
            "`contentHTML`" =>'<div style="font-family:arial; font-size:12px;">
                        {include file="string:{config name=emailheaderhtml}"}
                        <br/><br/>
                        <p>Dear {$billingaddress.salutation|salutation} {$billingaddress.lastname},<br/>
                            <br/>
                            Thank you for your order at {config name=shopName} (Number: {$sOrderNumber}) on {$sOrderDay} at {$sOrderTime}.<br/>
                            <br/>
                            <strong>Information on your order:</strong></p><br/>
                        <table width="80%" border="0" style="font-family:Arial, Helvetica, sans-serif; font-size:12px;">
                            <tr>
                                <td bgcolor="#F7F7F2" style="border-bottom:1px solid #cccccc;"><strong>Pos.</strong></td>
                                <td bgcolor="#F7F7F2" style="border-bottom:1px solid #cccccc;"><strong>Article</strong></td>
                                <td bgcolor="#F7F7F2" style="border-bottom:1px solid #cccccc;"><strong>Description</strong></td>
                                <td bgcolor="#F7F7F2" style="border-bottom:1px solid #cccccc;"><strong>Quantities</strong></td>
                                {if $attributes.bopis_shipping_status == 2}
                                    <td bgcolor="#F7F7F2" style="border-bottom:1px solid #cccccc;"><strong>Shipping Method</strong></td>
                                {/if}
                                <td bgcolor="#F7F7F2" style="border-bottom:1px solid #cccccc;"><strong>Price</strong></td>
                                <td bgcolor="#F7F7F2" style="border-bottom:1px solid #cccccc;"><strong>Total</strong></td>
                            </tr>
                    
                            {foreach item=details key=position from=$sOrderDetails}
                            <tr>
                                <td style="border-bottom:1px solid #cccccc;">{$position+1|fill:4} </td>
                                <td style="border-bottom:1px solid #cccccc;">{if $details.image.src.0 && $details.modus == 0}<img style="height: 57px;" height="57" src="{$details.image.src.0}" alt="{$details.articlename}" />{else} {/if}</td>
                                <td style="border-bottom:1px solid #cccccc;">
                                    {$details.articlename|wordwrap:80|indent:4}<br>
                                    Article-No: {$details.ordernumber|fill:20}<br>
                                    {if $details.__s_order_basket_attributes_instorepickup == "1" || $details.__s_order_basket_attributes_curbsidepick == "1"}
                                        Store: {$details.__s_order_basket_attributes_businessname|wordwrap:80|indent:4}
                                    {/if}
                                </td>
                                <td style="border-bottom:1px solid #cccccc;">{$details.quantity|fill:6}</td>
                                {if $attributes.bopis_shipping_status == 2}
                                    <td style="border-bottom:1px solid #cccccc;">
                                        {$details.__s_order_basket_attributes_bopis_shipping_name|wordwrap:80|indent:4}<br>
                                        {$details.__s_order_basket_attributes_bopis_shipping_fee|padding:8|currency}
                                    </td>
                                {/if}
                                <td style="border-bottom:1px solid #cccccc;">{$details.price|padding:8|currency}</td>
                                <td style="border-bottom:1px solid #cccccc;">{$details.amount|padding:8|currency}</td>
                            </tr>
                            {/foreach}
                    
                        </table>
                    
                        <p>
                            <br/>
                            <br/>
                            Shipping costs: {$sShippingCosts|currency}<br/>
                            Net total: {$sAmountNet|currency}<br/>
                            {if !$sNet}
                            {foreach $sTaxRates as $rate => $value}
                            plus {$rate|number_format:0}% VAT {$value|currency}<br/>
                            {/foreach}
                            <strong>Total gross: {$sAmount|currency}</strong><br/>
                            {/if}
                            <br/>
                            <br/>
                            <strong>Selected payment type:</strong> {$additional.payment.description}<br/>
                            {$additional.payment.additionaldescription}
                            {if $additional.payment.name == "debit"}
                            Your bank connection:<br/>
                            Account number: {$sPaymentTable.account}<br/>
                            BIN: {$sPaymentTable.bankcode}<br/>
                            Bank name: {$sPaymentTable.bankname}<br/>
                            Bank holder: {$sPaymentTable.bankholder}<br/>
                            <br/>
                            We will withdraw the money from your bank account within the next days.<br/>
                            {/if}
                            <br/>
                            <br/>
                            {if $additional.payment.name == "prepayment"}
                            Our bank connection:<br/>
                            Account: ###<br/>
                            BIN: ###<br/>
                            {/if}
                            <br/>
                            <br/>
                            <strong>Selected shipping type:</strong> {$sDispatch.name}<br/>
                            {$sDispatch.description}
                        </p>
                        <br/>
                        <p>
                            {if $sComment}
                            <strong>Your comment:</strong><br/>
                            {$sComment}<br/>
                            {/if}
                            <br/>
                            <br/>
                            <strong>Billing address:</strong><br/>
                            {$billingaddress.company}<br/>
                            {$billingaddress.firstname} {$billingaddress.lastname}<br/>
                            {$billingaddress.street} {$billingaddress.streetnumber}<br/>
                            {if {config name=showZipBeforeCity}}{$billingaddress.zipcode} {$billingaddress.city}{else}{$billingaddress.city} {$billingaddress.zipcode}{/if}<br/>
                            {$additional.country.countryname}<br/>
                            <br/>
                            <br/>
                            <strong>Shipping address:</strong><br/>
                            {$shippingaddress.company}<br/>
                            {$shippingaddress.firstname} {$shippingaddress.lastname}<br/>
                            {$shippingaddress.street} {$shippingaddress.streetnumber}<br/>
                            {if {config name=showZipBeforeCity}}{$shippingaddress.zipcode} {$shippingaddress.city}{else}{$shippingaddress.city} {$shippingaddress.zipcode}{/if}<br/>
                            {$additional.countryShipping.countryname}<br/>
                            <br/>
                            {if $billingaddress.ustid}
                            Your VAT-ID: {$billingaddress.ustid}<br/>
                            In case of a successful order and if you are based in one of the EU countries, you will receive your goods exempt from turnover tax.<br/>
                            {/if}
                            <br/>
                            <br/>
                            If you have any questions, do not hesitate to contact us.<br/>
                            {include file="string:{config name=emailfooterhtml}"}
                        </p>
                    </div>',
            "`ishtml`" => 1,
            "`attachment`" => '',
            "`mailtype`" => 2,
            "`name`"=> 'QuiversBOPISsOrder',
            "`context`" => $defaultContext[0]['context']
        ];
        $this->connection->insert("s_core_config_mails", $data);     

    }
}


