<?php

namespace QuiversBopis\Subscriber;

use Enlight\Event\SubscriberInterface;
use QuiversBopis\Components\OrderStorage;
use QuiversBopis\Components\ConfigHelper;
use Doctrine\DBAL\Connection;
use Quivers\Bootstrap\Database;
use Enlight_Controller_ActionEventArgs as ActionEventArgs;

class Frontend implements SubscriberInterface
{
    private $orderStorage;
    
    /**
     * @var Connection
     */
    private $connection;

    public function __construct(ConfigHelper $configHelper, Connection $connection, OrderStorage $orderStorage)
    {
  
       $this->configHelper = $configHelper;
       $this->connection = $connection;
       $this->session = Shopware()->Session();
       $this->basket = Shopware()->Modules()->Basket();
       $this->db =  Shopware()->Db();
       $this->orderStorage = $orderStorage;


    }

    /**
     * {@inheritdoc}
     */
    public static function getSubscribedEvents()
    {
        return [
            'Enlight_Controller_Action_PostDispatchSecure_Frontend' => 'quiversBopisAction',
            'Shopware_Modules_Basket_GetBasket_FilterSQL' => 'quiversLoadBasket',
            'Shopware_Modules_Basket_GetBasket_FilterResult'=>'quiversBasketData',
            'Shopware_Modules_Order_SaveOrder_ProcessDetails'=>'ordersStorage',
            'Shopware_Modules_Order_SendMail_FilterVariables'=>'emailDispatchDiscription',
            'Shopware_Modules_Order_SaveOrder_FilterAttributes'=>'updateStatusAttribute',
            'Shopware_Modules_Order_SendMail_Create' => 'setQuiversEmailTemplate'
        ];
    }


    public function setQuiversEmailTemplate(\Enlight_Event_EventArgs $args)
    {
        if ($this->configHelper->BopisActive() && $this->configHelper->BopisEmailFlag() ){
            $context = $args->context;
            $mail = Shopware()->TemplateMail()->createMail('QuiversBOPISsOrder', $context);   
            return $mail;         
        }
        else{
            return;
        }
    }

    public function updateStatusAttribute(\Enlight_Event_EventArgs $args)
    {
        $attributes = $args->getReturn();
        
        if (!array_key_exists('bopis_shipping_status', $attributes)){
            $subject = $args->subject;
            $basketData  = $subject->sBasketData;
            $quiversAttributes = [
                'bopis_shipping_status' => $basketData['bopis_shipping_status']
            ];
            $attributes = array_merge($attributes, $quiversAttributes);

        }
        return $attributes;

    }

    public function ordersStorage(\Enlight_Event_EventArgs $args)
    {
        if(!$this->configHelper->BopisActive()){
            return;
        }
        $this->orderStorage->orderstore($args);

    }

    public function quiversBopisAction(ActionEventArgs $args)
    {
        $controller = $args->getSubject();
        $view = $controller->View();
        $request = $controller->Request();
        $controllerName = $request->getControllerName();
        $action = $controller->Request()->getActionName();

        $bopisActive = $this->configHelper->BopisActive();
        $view->assign('QuiverBopisActive', $bopisActive); 
        $view->assign('QuiversMarketplace', $this->configHelper->getMarketPlaceForCurrency());   
        $view->assign('QuiversAPIBaseURL', $this->configHelper->getAPIbaseURL());   
        $view->assign('QuiversSiteBaseURL',  $this->configHelper->getsiteBaseUrl());   
        $view->assign('QuiversCDN',  $this->configHelper->getCDN());   
        
        //get all the shipping methods
        if($controllerName =="checkout" && $action!="finish"){
            $this->configHelper->getShippingMethods($args);
        }
        
        
        //For Product Detail page
        if($bopisActive && $controllerName =="detail" && $action=="index"){
            $sArticle = $view->getAssign('sArticle');
            $searchterm = $this->configHelper->getSearchTerm($sArticle);    
            $isSealable =$this->configHelper->isSaleable($sArticle);
            $searchterm = isset($isSealable['searchterm'])?$isSealable['searchterm']:null;
            $view->assign('QuiversSearchterm', $searchterm); 
            $view->assign('QuiversStores',  isset($isSealable['stores'])?json_encode($isSealable['stores']):null); 
            $view->assign('IsSaleable', $isSealable);
        } 

        if($bopisActive && $controllerName =="register" && $action=="index"){
            $this->registerEvent($view);
        }

        //When Product is added to the cart
        if ($bopisActive && $controllerName =="checkout" && $action=="ajaxCart"){
            $this->configHelper->addProdBopisData($args);
        }

        //For cart page
        if ($controllerName =="checkout" && $action == "cart"){
            $view->assign('QuiverssTarget',"cart"); 
            if($bopisActive){
                $this->configHelper->addProdStores($args, true);
            }  
            $this->configHelper->setShippingMethod($args);
        }

        //For adding stores in item when order confirm page is called
        if ($controllerName =="checkout" && $action == "confirm"){
            $view->assign('QuiverssTarget',"confirm");   
            if($bopisActive){
                $this->configHelper->addProdStores($args);
            }
            $this->configHelper->setShippingMethod($args);
        }

        //For Shipping payment page
        $allowedActions = ["shippingPayment"];
        if ($controllerName =="checkout" && in_array($action, $allowedActions, true)) {
            $this->configHelper->setShippingMethod($args);
        }

        //For finish page(when order is completed)
        $allowedActions = ["finish"];
        if ($controllerName =="checkout" && in_array($action, $allowedActions, true)) {
            $this->configHelper->setShippingMethod($args);

        }        
    }


    public function quiversLoadBasket()
    {
        $fieldHelper = Shopware()->Container()->get('shopware_storefront.field_helper_dbal');
        $attrs = $fieldHelper->getTableFields('s_order_basket_attributes', 's_order_basket_attributes');
        $sql = "
        SELECT
            s_order_basket.*,
            COALESCE (NULLIF(ad.packunit, ''), mad.packunit) AS packunit,
            a.main_detail_id AS mainDetailId,
            ad.id AS articleDetailId,
            ad.minpurchase,
            a.taxID,
            ad.instock AS instock,
            ad.suppliernumber,
            ad.maxpurchase,
            ad.purchasesteps,
            ad.purchaseunit,
            COALESCE (ad.unitID, mad.unitID) AS unitID,
            ad.laststock,
            ad.shippingtime,
            ad.releasedate,
            ad.releasedate AS sReleaseDate,
            COALESCE (ad.ean, mad.ean) AS ean,
            ad.stockmin,
            s_order_basket_attributes.attribute1 as ob_attr1,
            s_order_basket_attributes.attribute2 as ob_attr2,
            s_order_basket_attributes.attribute3 as ob_attr3,
            s_order_basket_attributes.attribute4 as ob_attr4,
            s_order_basket_attributes.attribute5 as ob_attr5,
            s_order_basket_attributes.attribute6 as ob_attr6,
           " . implode(',', $attrs) . '
        FROM s_order_basket
        LEFT JOIN s_articles_details AS ad ON ad.ordernumber = s_order_basket.ordernumber
        LEFT JOIN s_articles a ON (a.id = ad.articleID)
        LEFT JOIN s_articles_details AS mad ON mad.id = a.main_detail_id
        LEFT JOIN s_order_basket_attributes ON s_order_basket.id = s_order_basket_attributes.basketID
        WHERE sessionID=?
        ORDER BY id ASC, datum DESC
        ';
        return $sql;
    }


    public function quiversBasketData(\Enlight_Event_EventArgs $args){
        $basket = $args->getReturn();
        $status = $this->configHelper::shippingSatusCode;
        $inStore = 0;
        $shipping= 0; 
        $shippedItems = 0;
        foreach($basket['content'] as $basketItem){
            if($basketItem['__s_order_basket_attributes_instorepickup']=='1' || $basketItem['__s_order_basket_attributes_curbsidepick']=='1'){
                $inStore = 1;
            }
            else{
                $shipping = 1;
                $shippedItems++;
            }
        }
        if($inStore && $shipping){
            $status = $this->configHelper::mixedStatusCode;
        }
        elseif($inStore==1 && $shipping==0){
            $status = $this->configHelper::bopisStatusCode;
        }
        $bopisStatus = [
            "bopis_shipping_status" => $status,
            "quivers_shipped_items" =>$shippedItems
        ];
        $result = array_merge(
           $basket,
           $bopisStatus
        );
        return $result;
    }

    public function emailDispatchDiscription(\Enlight_Event_EventArgs $args){
        $variables = $args->getReturn();
        $subject = $args->getSubject();
        if(!$this->configHelper->BopisActive()){
            return $variables;
        }
        if ($variables['sDispatch'] && $this->session['QuiversShippingTitle']!= '' && $this->session['QuiversShippingTitle']!= null) {
            $variables['sDispatch']['name']=$variables['sDispatch']['name'].$this->session['QuiversShippingTitle'];  
        }
        if ($this->session['QuiversTotalShipping']!= 0 && $this->session['QuiversTotalShipping']!= null) {
            $articles = Shopware()->Modules()->Articles();
            $currency  =$subject->sBasketData['sCurrencyName']     ;
            $variables['sShippingCosts'] = $articles->sFormatPrice($this->session['QuiversTotalShipping']) . ' ' . $subject->sBasketData['sCurrencyName'];
        }

        return  $variables;
    }


    public function registerEvent($view){
        $countBasket = Shopware()->Modules()->Basket()->sCountBasket();
        $statusCode = 0;
        $basket = Shopware()->Modules()->Basket()->sGetBasket();
        $view->assign("quivers_store_note", $this->configHelper::storepickupMessage);
        if(!$countBasket){
            $view->assign("quivers_bopis_status", 0);
        }
        else{
            $view->assign("quivers_bopis_status", 0);
            $inStore = 0;
            $shipping= 0; 
            foreach($basket['content'] as $basketItem){
                if($basketItem['__s_order_basket_attributes_businessrefid']!=null && $basketItem['__s_order_basket_attributes_businessrefid']!=''){
                    $inStore = 1;
                }
                else{
                    $shipping = 1;
                }
            }
            if($inStore && $shipping){
                $view->assign("quivers_bopis_status", $this->configHelper::mixedStatusCode);
            }
            elseif($inStore==1 && $shipping==0){
                $view->assign("quivers_bopis_status", $this->configHelper::bopisStatusCode);
            }
        }
    }

}